VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "cHotKey"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
  Option Explicit
  ' demo project showing how to use the SendMessage API function and
  ' the WM_SETHOTKEY message to set a global hotkey for a VB form
  ' by Bryan Stafford of New Vision Software - newvision@mvps.org
  ' this demo is released into the public domain "as is" without
  ' warranty or guaranty of any kind.  In other words, use at
  ' your own risk.
  '
  ' thanks go to Ed Stegman for the idea and the preliminary code for
  ' this demo!
  '

  ' NOTE: the documentation for the WM_SETHOTKEY message in the Win32 SDK
  ' is incorrect.  the documentation states that the wParam passed in the
  ' message call should contain the keycode for the hotkey in the low-order
  ' WORD of the wParam and the modifiers in the high-order WORD.  the wParam
  ' actually need to contain the keycode in the low-order BYTE of the low-
  ' order WORD and the modifiers in the high-order BYTE of the low-order word.
  ' the high-order WORD of the wParam is ignored.
  
  
  ' RULES FOR GLOBAL HOTKEYS:
  '   only one hotkey per window.
  '   child windows may not have hotkeys.
  '   ESCAPE, SPACE, and TAB are invalid hot keys.
  '   when the hotkey is pressed, the system will bring the window's last
  '     active popup (if it exists) or the window itself (if there is no
  '     popup window) to the foreground.
  '   if the window already has a hotkey associated with it, the new hotkey
  '     replaces the old one.
  '   if more than one window has the same hotkey, the window that is
  '     activated by the hotkey is random.
  
  
  Public Enum HOTKEY_MODIFIERS
    HOTKEYF_SHIFT = 1&
    HOTKEYF_CONTROL = 2&
    HOTKEYF_ALT = 4&
    HOTKEYF_EXT = 8&
  End Enum
  
  Public Enum HOTKEY_RETURN_VALS
    [Invalid KeyCode] = (-1&)
    [Invalid hWnd] = 0&
    Success = 1&
    [Duplicate HotKey] = 2&
  End Enum
  
  Private Const WM_SETHOTKEY As Long = &H32&
  
  Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, wParam As Any, lParam As Any) As Long

  Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (lpDest As Any, lpSource As Any, ByVal cBytes&)

Public Function SetHotKey(ByVal hWnd&, ByVal nKeyCode As VBRUN.KeyCodeConstants, ByVal fModifiers As HOTKEY_MODIFIERS) As HOTKEY_RETURN_VALS
  ' possible return values:
  ' 1 - The function is unsuccessfulthe hot key is invalid.
  '  0 - The function is unsuccessfulthe window is invalid.
  '  1 - The function is successful, and no other window has the same hot key.
  '  2 - The function is successful, but another window already has the same hot key.

  Dim fHotKey&, nRet&
    
  ' assign the keycode to the low-order BYTE of the variable
  fHotKey = nKeyCode
  
  ' copy the modifiers into the high-order BYTE of the low-order WORD of the variable
  CopyMemory ByVal VarPtr(fHotKey) + 1, ByVal VarPtr(fModifiers), 1&
  
  ' set the hotkey
  nRet = SendMessage(hWnd, WM_SETHOTKEY, ByVal fHotKey, ByVal 0&)
  
  ' don't allow duplicate keys.  remove the hotkey if it is a dupelicate.
  If nRet = [Duplicate HotKey] Then Call KillHotKey(hWnd)
  
  ' return the return value from the sendmessage call.
  SetHotKey = nRet
    
End Function

Public Function KillHotKey(ByVal hWnd&) As Long
  ' passing NULL in the wParam of the sendmessage call will clear the hotkey for a window
  KillHotKey = SendMessage(hWnd, WM_SETHOTKEY, ByVal 0&, ByVal 0&)
End Function
